@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013 Oracle Corporation
+ * Copyright (C) 2013-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -87,12 +87,14 @@
 #define CURSOR_PIXEL_COUNT VBOX_MAX_CURSOR_WIDTH * VBOX_MAX_CURSOR_HEIGHT
 #define CURSOR_DATA_SIZE CURSOR_PIXEL_COUNT * 4 + CURSOR_PIXEL_COUNT / 8
 
+#define VBOX_MAX_SCREENS  32
+
 struct vbox_fbdev;
 
 struct vbox_private {
     struct drm_device *dev;
 
-    uint8_t __iomem *vram;
+    uint8_t __iomem *mapped_vram;
     HGSMIGUESTCOMMANDCONTEXT submit_info;
     struct VBVABUFFERCONTEXT *vbva_info;
     bool any_pitch;
@@ -101,7 +103,9 @@ struct vbox_private {
     /** Amount of available VRAM, including space used for buffers. */
     uint32_t full_vram_size;
     /** Amount of available VRAM, not including space used for buffers. */
-    uint32_t vram_size;
+    uint32_t available_vram_size;
+    /** Offset of mapped VRAM area in full VRAM. */
+    uint32_t vram_map_start;
     /** Offset to the host flags in the VRAM. */
     uint32_t host_flags_offset;
     /** Array of structures for receiving mode hints. */
@@ -227,6 +231,7 @@ int vbox_framebuffer_init(struct drm_device *dev,
 int vbox_fbdev_init(struct drm_device *dev);
 void vbox_fbdev_fini(struct drm_device *dev);
 void vbox_fbdev_set_suspend(struct drm_device *dev, int state);
+void vbox_fbdev_set_base(struct vbox_private *vbox, unsigned long gpu_addr);
 
 struct vbox_bo {
     struct ttm_buffer_object bo;
@@ -266,7 +271,7 @@ extern int vbox_dumb_mmap_offset(struct drm_file *file,
                 uint32_t handle,
                 uint64_t *offset);
 
-#define DRM_FILE_PAGE_OFFSET (0x100000000ULL >> PAGE_SHIFT)
+#define DRM_FILE_PAGE_OFFSET (0x10000000ULL >> PAGE_SHIFT)
 
 int vbox_mm_init(struct vbox_private *vbox);
 void vbox_mm_fini(struct vbox_private *vbox);
@@ -285,7 +290,11 @@ static inline int vbox_bo_reserve(struct vbox_bo *bo, bool no_wait)
 {
     int ret;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 7, 0)
+    ret = ttm_bo_reserve(&bo->bo, true, no_wait, NULL);
+#else
     ret = ttm_bo_reserve(&bo->bo, true, no_wait, false, 0);
+#endif
     if (ret)
     {
         if (ret != -ERESTARTSYS && ret != -EBUSY)
