@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2013 Oracle Corporation
+ * Copyright (C) 2013-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -106,7 +106,8 @@ static void vbox_do_modeset(struct drm_crtc *crtc,
                                 crtc->x * bpp / 8 + crtc->y * pitch,
                                 pitch, width, height,
                                 vbox_crtc->blanked ? 0 : bpp, flags);
-    VBoxHGSMIReportFlagsLocation(&vbox->submit_info, vbox->host_flags_offset);
+    VBoxHGSMIReportFlagsLocation(&vbox->submit_info,   vbox->vram_map_start
+                                                     + vbox->host_flags_offset);
     LogFunc(("vboxvideo: %d\n", __LINE__));
 }
 
@@ -132,9 +133,9 @@ static int vbox_set_view(struct drm_crtc *crtc)
         VBVAINFOVIEW *pInfo = (VBVAINFOVIEW *)p;
         pInfo->u32ViewIndex = vbox_crtc->crtc_id;
         pInfo->u32ViewOffset = vbox_crtc->fb_offset;
-        pInfo->u32ViewSize =   vbox->vram_size - vbox_crtc->fb_offset
+        pInfo->u32ViewSize =   vbox->available_vram_size - vbox_crtc->fb_offset
                              + vbox_crtc->crtc_id * VBVA_MIN_BUFFER_SIZE;
-        pInfo->u32MaxScreenSize = vbox->vram_size - vbox_crtc->fb_offset;
+        pInfo->u32MaxScreenSize = vbox->available_vram_size - vbox_crtc->fb_offset;
         VBoxHGSMIBufferSubmit(&vbox->submit_info, p);
         VBoxHGSMIBufferFree(&vbox->submit_info, p);
     }
@@ -224,6 +225,8 @@ static int vbox_crtc_do_set_base(struct drm_crtc *crtc,
         ret = ttm_bo_kmap(&bo->bo, 0, bo->bo.num_pages, &bo->kmap);
         if (ret)
             DRM_ERROR("failed to kmap fbcon\n");
+        else
+            vbox_fbdev_set_base(vbox, gpu_addr);
     }
     vbox_bo_unreserve(bo);
 
@@ -688,6 +691,11 @@ static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
     size_t data_size, mask_size;
     bool src_isiomem;
 
+    /* Re-set this regularly as in 5.0.20 and earlier the information was lost
+     * on save and restore. */
+    VBoxHGSMIUpdateInputMapping(&vbox->submit_info, 0, 0,
+                                vbox->input_mapping_width,
+                                vbox->input_mapping_height);
     if (!handle) {
         bool cursor_enabled = false;
         struct drm_crtc *crtci;
@@ -714,7 +722,11 @@ static int vbox_cursor_set2(struct drm_crtc *crtc, struct drm_file *file_priv,
         || !(caps & VMMDEV_MOUSE_HOST_WANTS_ABSOLUTE))
         return -EINVAL;
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 7, 0)
+    obj = drm_gem_object_lookup(file_priv, handle);
+#else
     obj = drm_gem_object_lookup(crtc->dev, file_priv, handle);
+#endif
     if (obj)
     {
         bo = gem_to_vbox_bo(obj);
@@ -783,7 +795,8 @@ static int vbox_cursor_move(struct drm_crtc *crtc,
         return 0;
     rc = VBoxHGSMICursorPosition(&vbox->submit_info, true, x + crtc->x,
                                  y + crtc->y, &host_x, &host_y);
-    if (RT_FAILURE(rc))
+    /* Work around a bug after save and restore in 5.0.20 and earlier. */
+    if (RT_FAILURE(rc) || (host_x == 0 && host_y == 0))
         return -RTErrConvertToErrno(rc);
     if (x + crtc->x < host_x)
         hot_x = min(host_x - x - crtc->x, vbox->cursor_width);
