@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Linux Additions X11 setup init script ($Revision: 106429 $)
+# Linux Additions X11 setup init script ($Revision: 107150 $)
 #
 
 #
@@ -32,6 +32,7 @@ PATH=$PATH:/bin:/sbin:/usr/sbin
 LOG="/var/log/vboxadd-install-x11.log"
 CONFIG_DIR="/var/lib/VBoxGuestAdditions"
 CONFIG="config"
+SCRIPTNAME=vboxadd-x11.sh
 MODPROBE=/sbin/modprobe
 
 if $MODPROBE -c 2>/dev/null | grep -q '^allow_unsupported_modules  *0'; then
@@ -73,79 +74,27 @@ x11conf_files="/etc/X11/xorg.conf /etc/X11/xorg.conf-4 /etc/X11/.xorg.conf \
     /usr/X11R6/etc/X11/XF86Config /usr/X11R6/lib/X11/XF86Config-4 \
     /usr/X11R6/lib/X11/XF86Config"
 
-if [ -f /etc/redhat-release ]; then
-    system=redhat
-elif [ -f /etc/debian_version ]; then
-    system=debian
-elif [ -f /etc/SuSE-release ]; then
-    system=suse
-elif [ -f /etc/gentoo-release ]; then
-    system=gentoo
-else
-    system=other
+# Preamble for Gentoo
+if [ "`which $0`" = "/sbin/rc" ]; then
+    shift
 fi
 
-if [ "$system" = "redhat" ]; then
-    . /etc/init.d/functions
-    fail_msg() {
-        echo_failure
-        echo
-    }
-    succ_msg() {
-        echo_success
-        echo
-    }
-    begin() {
-        echo -n "$1"
-    }
-fi
+begin()
+{
+    test -n "${2}" && echo "${SCRIPTNAME}: ${1}."
+    logger -t "${SCRIPTNAME}" "${1}."
+}
 
-if [ "$system" = "suse" ]; then
-    . /etc/rc.status
-    fail_msg() {
-        rc_failed 1
-        rc_status -v
-    }
-    succ_msg() {
-        rc_reset
-        rc_status -v
-    }
-    begin() {
-        echo -n "$1"
-    }
-fi
+succ_msg()
+{
+    logger -t "${SCRIPTNAME}" "${1}."
+}
 
-if [ "$system" = "gentoo" ]; then
-    if [ -f /sbin/functions.sh ]; then
-        . /sbin/functions.sh
-    elif [ -f /etc/init.d/functions.sh ]; then
-        . /etc/init.d/functions.sh
-    fi
-    fail_msg() {
-        eend 1
-    }
-    succ_msg() {
-        eend $?
-    }
-    begin() {
-        ebegin $1
-    }
-    if [ "`which $0`" = "/sbin/rc" ]; then
-        shift
-    fi
-fi
-
-if [ "$system" = "debian" -o "$system" = "other" ]; then
-    fail_msg() {
-        echo " ...fail!"
-    }
-    succ_msg() {
-        echo " ...done."
-    }
-    begin() {
-        echo -n $1
-    }
-fi
+fail_msg()
+{
+    echo "${SCRIPTNAME}: ${1}." >&2
+    logger -t "${SCRIPTNAME}" "${1}."
+}
 
 dev=/dev/vboxguest
 userdev=/dev/vboxuser
@@ -154,12 +103,8 @@ group=1
 
 fail()
 {
-    if [ "$system" = "gentoo" ]; then
-        eerror $1
-        exit 1
-    fi
-    fail_msg
-    echo "($1)"
+    echo "${SCRIPTNAME}: failed: ${1}." >&2
+    logger "${SCRIPTNAME}: ${1}."
     exit 1
 }
 
@@ -428,6 +373,17 @@ setup()
             dox11config=""
             ;;
     esac
+    case "${x_version}" in
+    4.* | 6.* | 7.* | 1.?.* | 1.1[0-6].* )
+        echo "blacklist vboxvideo" > /etc/modprobe.d/blacklist-vboxvideo.conf
+        ;;
+    *)
+        if test -f /etc/modprobe.d/blacklist-vboxvideo.conf; then
+            rm -f /etc/modprobe.d/blacklist-vboxvideo.conf
+            ${MODPROBE} vboxvideo
+        fi
+        ;;
+    esac
     test -n "${dox11config}" &&
         begin "Installing $xserver_version modules"
     case "$vboxvideo_src" in
@@ -444,7 +400,7 @@ setup()
         *)
         rm "$modules_dir/input/vboxmouse_drv$driver_ext" 2>/dev/null
     esac
-    succ_msg
+    succ_msg "$xserver_version modules installed"
 
     if test -n "$dox11config"; then
         begin "Setting up the Window System to use the Guest Additions"
@@ -485,7 +441,7 @@ setup()
                 touch "${nobak_cfg}"
             fi
         fi
-        succ_msg
+        succ_msg "Window System set up to use the Guest Additions"
         test -n "$generated" &&
             cat << EOF
 The following X.Org/XFree86 configuration files were originally generated by
@@ -532,7 +488,7 @@ EOF
     install_x11_startup_app "$lib_dir/98vboxadd-xclient" "$share_dir/vboxclient.desktop" VBoxClient VBoxClient-all ||
         fail "See the log file $LOG for more information."
     ln -s "$lib_dir/98vboxadd-xclient" /usr/bin/VBoxClient-all 2>/dev/null
-    succ_msg
+    succ_msg "Window system drivers installed"
 }
 
 cleanup()
