@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2013 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -93,11 +93,8 @@
 #ifdef XORG_7X
 # include <stdlib.h>
 # include <string.h>
-# include <sys/stat.h>
-# define xf86stat stat
-# define xf86stat_s stat
-#else
-# include <xf86_ansic.h>
+# include <fcntl.h>
+# include <unistd.h>
 #endif
 
 /* Mandatory functions */
@@ -115,12 +112,15 @@ static Bool VBOXScreenInit(ScreenPtr pScreen, int argc, char **argv);
 static Bool VBOXEnterVT(ScrnInfoPtr pScrn);
 static void VBOXLeaveVT(ScrnInfoPtr pScrn);
 static Bool VBOXCloseScreen(ScreenPtr pScreen);
+#ifndef VBOXVIDEO_13
 static Bool VBOXSaveScreen(ScreenPtr pScreen, int mode);
+#endif
 static Bool VBOXSwitchMode(ScrnInfoPtr pScrn, DisplayModePtr pMode);
 static void VBOXAdjustFrame(ScrnInfoPtr pScrn, int x, int y);
 static void VBOXFreeScreen(ScrnInfoPtr pScrn);
-static void VBOXDisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode,
-                                          int flags);
+#ifndef VBOXVIDEO_13
+static void VBOXDisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode, int flags);
+#endif
 
 /* locally used functions */
 static Bool VBOXMapVidMem(ScrnInfoPtr pScrn);
@@ -289,12 +289,15 @@ static Bool adjustScreenPixmap(ScrnInfoPtr pScrn, int width, int height)
     return TRUE;
 }
 
-/** Set a video mode to the hardware, RandR 1.1 version.  Since we no longer do
- * virtual frame buffers, adjust the screen pixmap dimensions to match.  The
- * "override" parameters are for when we received a mode hint while switched to
- * a virtual terminal.  In this case VBoxClient will have told us about the
- * mode, but not yet been able to do a mode switch using RandR.  We solve this
- * by setting the requested mode to the host but keeping the virtual frame-
+#ifndef VBOXVIDEO_13
+/** Set a video mode to the hardware, RandR 1.1 version.
+ *
+ * Since we no longer do virtual frame buffers, adjust the screen pixmap
+ * dimensions to match.  The "override" parameters are for when we received a
+ * mode hint while switched to a virtual terminal.  In this case VBoxClient will
+ * have told us about the mode, but not yet been able to do a mode switch using
+ * RandR.  We solve this by setting the requested mode to the host but keeping
+ * the virtual frame-
  * buffer matching what the X server expects. */
 static void setModeRandR11(ScrnInfoPtr pScrn, DisplayModePtr pMode, bool fScreenInitTime, bool fEnterVTTime,
                            int cXOverRide, int cYOverRide)
@@ -331,6 +334,7 @@ static void setModeRandR11(ScrnInfoPtr pScrn, DisplayModePtr pMode, bool fScreen
         vbvxSetMode(pScrn, 0, cXPhysical, cYPhysical, 0, 0, true, true, &frameBuffer);
     pScrn->currentMode = pMode;
 }
+#endif
 
 #ifdef VBOXVIDEO_13
 /* X.org 1.3+ mode-setting support ******************************************/
@@ -551,7 +555,6 @@ static DisplayModePtr vbox_output_add_mode(VBOXPtr pVBox, DisplayModePtr *pModes
 static DisplayModePtr
 vbox_output_get_modes (xf86OutputPtr output)
 {
-    unsigned cIndex = 0;
     DisplayModePtr pModes = NULL;
     DisplayModePtr pPreferred = NULL;
     ScrnInfoPtr pScrn = output->scrn;
@@ -635,6 +638,7 @@ static pointer
 vboxSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
 {
     static Bool Initialised = FALSE;
+    RT_NOREF(Options, ErrorMinor);
 
     if (!Initialised)
     {
@@ -665,20 +669,24 @@ vboxSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
 static const OptionInfoRec *
 VBOXAvailableOptions(int chipid, int busid)
 {
+    RT_NOREF(chipid, busid);
     return (VBOXOptions);
 }
 
 static void
 VBOXIdentify(int flags)
 {
+    RT_NOREF(flags);
     xf86PrintChipsets(VBOX_NAME, "guest driver for VirtualBox", VBOXChipsets);
 }
 
 #ifndef XF86_SCRN_INTERFACE
 # define SCRNINDEXAPI(pfn) pfn ## Index
-static Bool VBOXScreenInitIndex(int scrnIndex, ScreenPtr pScreen, int argc,
-                                char **argv)
-{ return VBOXScreenInit(pScreen, argc, argv); }
+static Bool VBOXScreenInitIndex(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+{
+    RT_NOREF(scrnIndex);
+    return VBOXScreenInit(pScreen, argc, argv);
+}
 
 static Bool VBOXEnterVTIndex(int scrnIndex, int flags)
 { (void) flags; return VBOXEnterVT(xf86Screens[scrnIndex]); }
@@ -727,13 +735,15 @@ VBOXPciProbe(DriverPtr drv, int entity_num, struct pci_device *dev,
              intptr_t match_data)
 {
     ScrnInfoPtr pScrn;
-    struct xf86stat_s sstat;
+    int drmFd;
 
     TRACE_ENTRY();
 
-    if (xf86stat("/dev/dri/card0", &sstat) == 0)
+    drmFd = open("/dev/dri/card0", O_RDWR, 0);
+    if (drmFd >= 0)
     {
         xf86Msg(X_INFO, "vboxvideo: kernel driver found, not loading.\n");
+        close(drmFd);
         return FALSE;
     }
     pScrn = xf86ConfigPciEntity(NULL, 0, entity_num, VBOXPCIchipsets,
@@ -995,9 +1005,6 @@ vboxLoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
 static void updateGraphicsCapability(ScrnInfoPtr pScrn, Bool hasVT)
 {
     VBOXPtr pVBox = VBOXGetRec(pScrn);
-    size_t cData;
-    int32_t *paData;
-    int rc;
 
     if (!pVBox->fHaveHGSMIModeHints)
         return;
@@ -1029,8 +1036,7 @@ static void setSizesRandR11(ScrnInfoPtr pScrn)
 
 static void setSizesAndCursorIntegration(ScrnInfoPtr pScrn, bool fScreenInitTime)
 {
-    VBOXPtr pVBox = VBOXGetRec(pScrn);
-
+    RT_NOREF(fScreenInitTime);
     TRACE_LOG("fScreenInitTime=%d\n", (int)fScreenInitTime);
 #ifdef VBOXVIDEO_13
 # if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) >= 5
@@ -1053,7 +1059,6 @@ static void setSizesAndCursorIntegration(ScrnInfoPtr pScrn, bool fScreenInitTime
 static void vboxBlockHandler(pointer pData, OSTimePtr pTimeout, pointer pReadmask)
 {
     ScrnInfoPtr pScrn = (ScrnInfoPtr)pData;
-    VBOXPtr pVBox = VBOXGetRec(pScrn);
     bool fNeedUpdate = false;
 
     (void)pTimeout;
@@ -1083,6 +1088,7 @@ static Bool VBOXScreenInit(ScreenPtr pScreen, int argc, char **argv)
     ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     VBOXPtr pVBox = VBOXGetRec(pScrn);
     VisualPtr visual;
+    RT_NOREF(argc, argv);
 
     TRACE_ENTRY();
 
@@ -1207,10 +1213,10 @@ static Bool VBOXScreenInit(ScreenPtr pScreen, int argc, char **argv)
     if (!xf86SetDesiredModes(pScrn)) {
         return FALSE;
     }
-#else
+#else  /* !VBOXVIDEO_13 */
     /* set first video mode */
     setModeRandR11(pScrn, pScrn->currentMode, true, false, 0, 0);
-#endif
+#endif /* !VBOXVIDEO_13 */
 
     /* Say that we support graphics. */
     updateGraphicsCapability(pScrn, TRUE);
@@ -1286,8 +1292,8 @@ static Bool VBOXEnterVT(ScrnInfoPtr pScrn)
 
 static void VBOXLeaveVT(ScrnInfoPtr pScrn)
 {
-    VBOXPtr pVBox = VBOXGetRec(pScrn);
 #ifdef VBOXVIDEO_13
+    VBOXPtr pVBox = VBOXGetRec(pScrn);
     unsigned i;
 #else
     int32_t propertyValue = 0;
@@ -1421,12 +1427,14 @@ VBOXUnmapVidMem(ScrnInfoPtr pScrn)
     TRACE_EXIT();
 }
 
+#ifndef VBOXVIDEO_13
 static Bool
 VBOXSaveScreen(ScreenPtr pScreen, int mode)
 {
     (void)pScreen; (void)mode;
     return TRUE;
 }
+#endif
 
 void
 VBOXSaveMode(ScrnInfoPtr pScrn)
@@ -1461,9 +1469,10 @@ VBOXRestoreMode(ScrnInfoPtr pScrn)
         VBoxVideoDisableVBE();
 }
 
+#ifndef VBOXVIDEO_13
 static void
-VBOXDisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode,
-                int flags)
+VBOXDisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode, int flags)
 {
     (void)pScrn; (void)mode; (void) flags;
 }
+#endif
