@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -58,7 +58,7 @@ typedef struct
 #define bootp_clients ((BOOTPClient *)pData->pbootp_clients)
 
 /* XXX: only DHCP is supported */
-static const uint8_t rfc1533_cookie[] = { RFC1533_COOKIE };
+static const uint8_t rfc1533_cookie[4] = { RFC1533_COOKIE };
 
 static void bootp_reply(PNATState pData, struct mbuf *m0, int offReply, uint16_t flags);
 
@@ -66,7 +66,7 @@ static uint8_t *dhcp_find_option(uint8_t *vend, uint8_t tag)
 {
     uint8_t *q = vend;
     uint8_t len;
-    /*@todo magic validation */
+    /** @todo magic validation */
     q += 4; /*magic*/
     while(*q != RFC1533_END)
     {
@@ -362,6 +362,7 @@ static int dhcp_send_ack(PNATState pData, struct bootp_t *bp, BOOTPClient *bc, s
     int offReply = 0; /* boot_reply will fill general options and add END before sending response */
 
     dhcp_create_msg(pData, bp, m, DHCPACK);
+    slirp_update_guest_addr_guess(pData, bc->addr.s_addr, "DHCP ACK");
     offReply = dhcp_do_ack_offer(pData, m, bc, fDhcpRequest);
     return offReply;
 }
@@ -472,7 +473,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, struct mbuf
                 || req_ip
                 || bp->bp_ciaddr.s_addr == INADDR_ANY)
             {
-                LogRel(("NAT: invalid RENEWING dhcp request\n"));
+                LogRel(("NAT: Invalid RENEWING dhcp request\n"));
                 return -1; /* silent ignorance */
             }
             if (bc != NULL)
@@ -492,7 +493,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, struct mbuf
                bc = bc_alloc_client(pData);
                if (!bc)
                {
-                   LogRel(("NAT: can't alloc address. RENEW has been silently ignored.\n"));
+                   LogRel(("NAT: Can't allocate address. RENEW has been silently ignored\n"));
                    return -1;
                }
 
@@ -527,7 +528,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, struct mbuf
             ui32 = *(uint32_t *)(req_ip + 2);
             if ((ui32 & RT_H2N_U32(pData->netmask)) != pData->special_addr.s_addr)
             {
-                LogRel(("NAT: address %RTnaipv4 has been requested -- sending NAK\n", ui32));
+                LogRel(("NAT: Address %RTnaipv4 has been requested -- sending NAK\n", ui32));
                 offReply = dhcp_send_nack(pData, bp, bc, m);
                 return offReply;
             }
@@ -538,7 +539,7 @@ static int dhcp_decode_request(PNATState pData, struct bootp_t *bp, struct mbuf
                 bc = bc_alloc_client(pData);
                 if (!bc)
                 {
-                    LogRel(("NAT: can't alloc address. RENEW has been silently ignored\n"));
+                    LogRel(("NAT: Can't allocate address. RENEW has been silently ignored\n"));
                     return -1;
                 }
             }
@@ -590,21 +591,17 @@ static int dhcp_decode_discover(PNATState pData, struct bootp_t *bp, int fDhcpDi
         offReply = dhcp_send_offer(pData, bp, bc, m);
         return offReply;
     }
-    else
-    {
-        bc = find_addr(pData, &daddr, bp->bp_hwaddr);
-        if (!bc)
-        {
-            LogRel(("NAT: DHCP Inform was ignored no boot client was found\n"));
-            return -1;
-        }
 
-        LogRel(("NAT: DHCP offered IP address %RTnaipv4\n", bc->addr.s_addr));
-        offReply = dhcp_send_ack(pData, bp, bc, m, /* fDhcpRequest=*/ 0);
-        return offReply;
+    bc = find_addr(pData, &daddr, bp->bp_hwaddr);
+    if (!bc)
+    {
+        LogRel(("NAT: DHCP Inform was ignored no boot client was found\n"));
+        return -1;
     }
 
-    return -1;
+    LogRel(("NAT: DHCP offered IP address %RTnaipv4\n", bc->addr.s_addr));
+    offReply = dhcp_send_ack(pData, bp, bc, m, /* fDhcpRequest=*/ 0);
+    return offReply;
 }
 
 static int dhcp_decode_release(PNATState pData, struct bootp_t *bp)
@@ -615,6 +612,7 @@ static int dhcp_decode_release(PNATState pData, struct bootp_t *bp)
             bp->bp_ciaddr.s_addr));
     return 0;
 }
+
 /**
  * fields for discovering t
  * Field      DHCPDISCOVER          DHCPREQUEST           DHCPDECLINE,
@@ -739,7 +737,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
     m = m_getcl(pData, M_DONTWAIT, MT_HEADER, M_PKTHDR);
     if (!m)
     {
-        LogRel(("NAT: can't alocate memory for response!\n"));
+        LogRel(("NAT: Can't allocate memory for response!\n"));
         return;
     }
 
@@ -785,7 +783,7 @@ static void dhcp_decode(PNATState pData, struct bootp_t *bp, const uint8_t *buf,
                 Assert(bc);
                 if (!bc)
                 {
-                    LogRel(("NAT: can't allocate bootp client object\n"));
+                    LogRel(("NAT: Can't allocate bootp client object\n"));
                     break;
                 }
                 bc->addr.s_addr = req_ip.s_addr;
