@@ -110,7 +110,9 @@ int rcp_parse(struct rcp_state *state, const char *filename)
     char *pszSearchBuf;
     size_t cbSearchBuf;
     uint32_t flags;
+#ifdef RCP_ACCEPT_PORT /* OS X extention */
     uint32_t default_port = RTNETADDR_PORT_NA;
+#endif
     unsigned i;
     int rc;
 
@@ -244,7 +246,7 @@ int rcp_parse(struct rcp_state *state, const char *filename)
             if (NetAddr.enmType == RTNETADDRTYPE_INVALID)
             {
                 LogRel(("NAT: resolv.conf: bad nameserver address %s\n", tok));
-                continue;                
+                continue;
             }
 
 
@@ -432,7 +434,6 @@ static int
 rcp_address_trailer(char **ppszNext, PRTNETADDR pNetAddr, RTNETADDRTYPE enmType)
 {
     char *pszNext = *ppszNext;
-    uint16_t port;
     int rc = VINF_SUCCESS;
 
     if (*pszNext == '\0')
@@ -443,6 +444,8 @@ rcp_address_trailer(char **ppszNext, PRTNETADDR pNetAddr, RTNETADDRTYPE enmType)
 #ifdef RCP_ACCEPT_PORT /* OS X extention */
     else if (*pszNext == '.')
     {
+        uint16_t port;
+
         rc = RTStrToUInt16Ex(++pszNext, NULL, 10, &port);
         if (RT_SUCCESS(rc))
         {
@@ -473,7 +476,7 @@ static char *getToken(char *psz, char **ppszSavePtr)
             return NULL;
     }
 
-    while (*psz && *psz == ' ' && *psz == '\t')
+    while (*psz == ' ' || *psz == '\t')
         ++psz;
 
     if (*psz == '\0')
