@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -32,7 +32,7 @@
 #endif
 #include <VBox/vmm/pdmdrv.h>
 #if defined (RT_OS_WINDOWS)
-#include <iphlpapi.h>
+#include <iprt/win/iphlpapi.h>
 #include <icmpapi.h>
 #endif
 
@@ -114,7 +114,7 @@ DECLINLINE(bool) slirpSend2Home(PNATState pData, struct socket *pSo, const void
         struct socket *pNewSocket = soCloneUDPSocketWithForegnAddr(pData, pSo, pData->pInSockAddrHomeAddress[idxAddr].sin_addr);
         AssertReturn((pNewSocket, false));
         pData->pInSockAddrHomeAddress[idxAddr].sin_port = pSo->so_fport;
-        /* @todo: more verbose on errors,
+        /** @todo more verbose on errors,
          * @note: we shouldn't care if this send fail or not (we're in broadcast).
          */
         LogFunc(("send %d bytes to %RTnaipv4 from %R[natsock]\n", cbBuf, pData->pInSockAddrHomeAddress[idxAddr].sin_addr.s_addr, pNewSocket));
@@ -134,7 +134,7 @@ static void sorecvfrom_icmp_unix(PNATState, struct socket *);
 #endif /* !RT_OS_WINDOWS */
 
 void
-so_init()
+so_init(void)
 {
 }
 
@@ -162,7 +162,7 @@ solookup(struct socket *head, struct in_addr laddr,
  * insque() it into the correct linked-list
  */
 struct socket *
-socreate()
+socreate(void)
 {
     struct socket *so;
 
@@ -240,9 +240,10 @@ soread(PNATState pData, struct socket *so)
 {
     int n, nn, lss, total;
     struct sbuf *sb = &so->so_snd;
-    size_t len = sb->sb_datalen - sb->sb_cc;
+    u_int len = sb->sb_datalen - sb->sb_cc;
     struct iovec iov[2];
     int mss = so->so_tcpcb->t_maxseg;
+    int sockerr;
 
     STAM_PROFILE_START(&pData->StatIOread, a);
     STAM_COUNTER_RESET(&pData->StatIORead_in_1);
@@ -320,30 +321,39 @@ soread(PNATState pData, struct socket *so)
 #else
     nn = recv(so->s, iov[0].iov_base, iov[0].iov_len, (so->so_tcpcb->t_force? MSG_OOB:0));
 #endif
+    if (nn < 0)
+        sockerr = errno; /* save it, as it may be clobbered by logging */
+    else
+        sockerr = 0;
+
     Log2(("%s: read(1) nn = %d bytes\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, nn));
     Log2(("%s: so = %R[natsock] so->so_snd = %R[sbuf]\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, so, sb));
     if (nn <= 0)
     {
-        /*
-         * Special case for WSAEnumNetworkEvents: If we receive 0 bytes that
-         * _could_ mean that the connection is closed. But we will receive an
-         * FD_CLOSE event later if the connection was _really_ closed. With
-         * www.youtube.com I see this very often. Closing the socket too early
-         * would be dangerous.
-         */
-        int status;
-        unsigned long pending = 0;
-        status = ioctlsocket(so->s, FIONREAD, &pending);
-        if (status < 0)
-            Log(("NAT:%s: error in WSAIoctl: %d\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, errno));
-        if (nn == 0 && (pending != 0))
+        if (nn == 0) /* XXX: should this be inside #if defined(RT_OS_WINDOWS)? */
         {
-            SOCKET_UNLOCK(so);
-            STAM_PROFILE_STOP(&pData->StatIOread, a);
-            return 0;
+            /*
+             * Special case for WSAEnumNetworkEvents: If we receive 0 bytes that
+             * _could_ mean that the connection is closed. But we will receive an
+             * FD_CLOSE event later if the connection was _really_ closed. With
+             * www.youtube.com I see this very often. Closing the socket too early
+             * would be dangerous.
+             */
+            int status;
+            unsigned long pending = 0;
+            status = ioctlsocket(so->s, FIONREAD, &pending);
+            if (status < 0)
+                Log(("NAT:%s: error in WSAIoctl: %d\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, errno));
+            if (pending != 0)
+            {
+                SOCKET_UNLOCK(so);
+                STAM_PROFILE_STOP(&pData->StatIOread, a);
+                return 0;
+            }
         }
+
         if (   nn < 0
-            && soIgnorableErrorCode(errno))
+            && soIgnorableErrorCode(sockerr))
         {
             SOCKET_UNLOCK(so);
             STAM_PROFILE_STOP(&pData->StatIOread, a);
@@ -351,18 +361,18 @@ soread(PNATState pData, struct socket *so)
         }
         else
         {
-            int fUninitiolizedTemplate = 0;
-            fUninitiolizedTemplate = RT_BOOL((   sototcpcb(so)
+            int fUninitializedTemplate = 0;
+            fUninitializedTemplate = RT_BOOL((   sototcpcb(so)
                                               && (  sototcpcb(so)->t_template.ti_src.s_addr == INADDR_ANY
                                                  || sototcpcb(so)->t_template.ti_dst.s_addr == INADDR_ANY)));
             /* nn == 0 means peer has performed an orderly shutdown */
             Log2(("%s: disconnected, nn = %d, errno = %d (%s)\n",
-                  RT_GCC_EXTENSION __PRETTY_FUNCTION__, nn, errno, strerror(errno)));
+                  RT_GCC_EXTENSION __PRETTY_FUNCTION__, nn, sockerr, strerror(sockerr)));
             sofcantrcvmore(so);
-            if (!fUninitiolizedTemplate)
+            if (!fUninitializedTemplate)
                 tcp_sockclosed(pData, sototcpcb(so));
             else
-                tcp_drop(pData, sototcpcb(so), errno);
+                tcp_drop(pData, sototcpcb(so), sockerr);
             SOCKET_UNLOCK(so);
             STAM_PROFILE_STOP(&pData->StatIOread, a);
             return -1;
@@ -391,7 +401,7 @@ soread(PNATState pData, struct socket *so)
      * a close will be detected on next iteration.
      * A return of -1 wont (shouldn't) happen, since it didn't happen above
      */
-    if (n == 2 && nn == iov[0].iov_len)
+    if (n == 2 && (unsigned)nn == iov[0].iov_len)
     {
         int ret;
         ret = recv(so->s, iov[1].iov_base, iov[1].iov_len, 0);
@@ -529,7 +539,7 @@ sowrite(PNATState pData, struct socket *so)
 {
     int n, nn;
     struct sbuf *sb = &so->so_rcv;
-    size_t len = sb->sb_cc;
+    u_int len = sb->sb_cc;
     struct iovec iov[2];
 
     STAM_PROFILE_START(&pData->StatIOwrite, a);
@@ -633,19 +643,19 @@ sowrite(PNATState pData, struct socket *so)
     }
 
 #ifndef HAVE_READV
-    if (n == 2 && nn == iov[0].iov_len)
+    if (n == 2 && (unsigned)nn == iov[0].iov_len)
     {
         int ret;
         ret = send(so->s, iov[1].iov_base, iov[1].iov_len, 0);
         if (ret > 0)
             nn += ret;
-        STAM_STATS({
-            if (ret > 0 && ret != iov[1].iov_len)
-            {
-                STAM_COUNTER_INC(&pData->StatIOWrite_rest);
-                STAM_COUNTER_ADD(&pData->StatIOWrite_rest_bytes, (iov[1].iov_len - ret));
-            }
-        });
+# ifdef VBOX_WITH_STATISTICS
+        if (ret > 0 && ret != (ssize_t)iov[1].iov_len)
+        {
+            STAM_COUNTER_INC(&pData->StatIOWrite_rest);
+            STAM_COUNTER_ADD(&pData->StatIOWrite_rest_bytes, (iov[1].iov_len - ret));
+        }
+#endif
     }
     Log2(("%s: wrote(2) nn = %d bytes\n", RT_GCC_EXTENSION __PRETTY_FUNCTION__, nn));
 #endif
@@ -678,7 +688,7 @@ sowrite(PNATState pData, struct socket *so)
 void
 sorecvfrom(PNATState pData, struct socket *so)
 {
-    LogFlowFunc(("sorecvfrom: so = %lx\n", (long)so));
+    LogFlowFunc(("sorecvfrom: so = %p\n", so));
 
 #ifdef RT_OS_WINDOWS
     /* ping is handled with ICMP API in ip_icmpwin.c */
@@ -693,7 +703,7 @@ sorecvfrom(PNATState pData, struct socket *so)
     else
 #endif /* !RT_OS_WINDOWS */
     {
-        static uint8_t au8Buf[64 * 1024];
+        static char achBuf[64 * 1024];
 
         /* A "normal" UDP packet */
         struct sockaddr_in addr;
@@ -723,8 +733,8 @@ sorecvfrom(PNATState pData, struct socket *so)
         iov[0].iov_len = M_TRAILINGSPACE(m);
 
         /* large packets will spill into a temp buffer */
-        iov[1].iov_base = au8Buf;
-        iov[1].iov_len = sizeof(au8Buf);
+        iov[1].iov_base = achBuf;
+        iov[1].iov_len = sizeof(achBuf);
 
 #if !defined(RT_OS_WINDOWS)
         {
@@ -741,11 +751,12 @@ sorecvfrom(PNATState pData, struct socket *so)
 #else  /* RT_OS_WINDOWS */
         {
             DWORD nbytes; /* NB: can't use nread b/c of different size */
-            DWORD flags;
+            DWORD flags = 0;
             int status;
-
-            flags = 0;
-            status = WSARecvFrom(so->s, iov, 2, &nbytes, &flags,
+            AssertCompile(sizeof(WSABUF) == sizeof(struct iovec));
+            AssertCompileMembersSameSizeAndOffset(WSABUF, len, struct iovec, iov_len);
+            AssertCompileMembersSameSizeAndOffset(WSABUF, buf, struct iovec, iov_base);
+            status = WSARecvFrom(so->s, (WSABUF *)&iov[0], 2, &nbytes, &flags,
                                  (struct sockaddr *)&addr, &addrlen,
                                  NULL, NULL);
             if (status != SOCKET_ERROR)
@@ -763,7 +774,7 @@ sorecvfrom(PNATState pData, struct socket *so)
                 m->m_len = iov[0].iov_len;
                 m_append(pData, m, nread - iov[0].iov_len, iov[1].iov_base);
             }
-            Assert((m_length(m, NULL) == nread));
+            Assert(m_length(m, NULL) == (size_t)nread);
 
             /*
              * Hack: domain name lookup will be used the most for UDP,
@@ -838,7 +849,7 @@ sosendto(PNATState pData, struct socket *so, struct mbuf *m)
     caddr_t buf = 0;
     int mlen;
 
-    LogFlowFunc(("sosendto: so = %R[natsock], m = %lx\n", so, (long)m));
+    LogFlowFunc(("sosendto: so = %R[natsock], m = %p\n", so, m));
 
     memset(&addr, 0, sizeof(struct sockaddr));
 #ifdef RT_OS_DARWIN
@@ -1336,7 +1347,7 @@ static void sorecvfrom_icmp_unix(PNATState pData, struct socket *so)
         else if (errno == ENETUNREACH)
             code = ICMP_UNREACH_NET;
 
-        LogRel((" udp icmp rx errno = %d (%s)\n", errno, strerror(errno)));
+        LogRel(("NAT: UDP ICMP rx errno=%d (%s)\n", errno, strerror(errno)));
         icmp_error(pData, so->so_m, ICMP_UNREACH, code, 0, strerror(errno));
         so->so_m = NULL;
         Log(("sorecvfrom_icmp_unix: 1 - step can't read IP datagramm\n"));
