@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2005-2013 Oracle Corporation
+ * Copyright (C) 2005-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,7 +17,7 @@
 
 #if !defined (VBOX_WITH_XPCOM)
 
-# include <objbase.h>
+# include <iprt/win/objbase.h>
 
 #else /* !defined (VBOX_WITH_XPCOM) */
 # include <stdlib.h>
@@ -55,7 +55,7 @@
 #include <VBox/version.h>
 
 #if !defined(RT_OS_DARWIN) && !defined(RT_OS_WINDOWS)
-char szXdgConfigHome[RTPATH_MAX] = "";
+char g_szXdgConfigHome[RTPATH_MAX] = "";
 #endif
 
 /**
@@ -66,13 +66,13 @@ char szXdgConfigHome[RTPATH_MAX] = "";
  * create a folder corresponding to the last in the list (the least
  * legacy) if none do.
  */
-const char *const apcszUserHome[] =
+const char * const g_apcszUserHome[] =
 #ifdef RT_OS_DARWIN
 { "Library/VirtualBox" };
 #elif defined RT_OS_WINDOWS
 { ".VirtualBox" };
 #else
-{ ".VirtualBox", szXdgConfigHome };
+{ ".VirtualBox", g_szXdgConfigHome };
 #endif
 
 #include "Logging.h"
@@ -232,25 +232,16 @@ int GetVBoxUserHomeDirectory(char *aDir, size_t aDirLen, bool fCreateDir)
         else
         {
 #if !defined(RT_OS_WINDOWS) && !defined(RT_OS_DARWIN)
-            const char *pcszConfigHome = RTEnvGet("XDG_CONFIG_HOME");
-            if (pcszConfigHome && pcszConfigHome[0])
-            {
-                vrc = RTStrCopy(szXdgConfigHome,
-                                sizeof(szXdgConfigHome),
-                                pcszConfigHome);
-                if (RT_SUCCESS(vrc))
-                    vrc = RTPathAppend(szXdgConfigHome,
-                                       sizeof(szXdgConfigHome),
-                                       "VirtualBox");
-            }
-            else
-                vrc = RTStrCopy(szXdgConfigHome,
-                                sizeof(szXdgConfigHome),
-                                ".config/VirtualBox");
+            vrc = RTEnvGetEx(RTENV_DEFAULT, "XDG_CONFIG_HOME", g_szXdgConfigHome, sizeof(g_szXdgConfigHome), NULL);
+            if (RT_SUCCESS(vrc))
+                vrc = RTPathAppend(g_szXdgConfigHome, sizeof(g_szXdgConfigHome), "VirtualBox");
+            AssertMsg(vrc == VERR_ENV_VAR_NOT_FOUND, ("%Rrc\n", vrc));
+            if (RT_FAILURE_NP(vrc))
+                vrc = RTStrCopy(g_szXdgConfigHome, sizeof(g_szXdgConfigHome), ".config/VirtualBox");
 #endif
-            for (unsigned i = 0; i < RT_ELEMENTS(apcszUserHome); ++i)
+            for (unsigned i = 0; i < RT_ELEMENTS(g_apcszUserHome); ++i)
             {
-                vrc = composeHomePath(aDir, aDirLen, apcszUserHome[i]);
+                vrc = composeHomePath(aDir, aDirLen, g_apcszUserHome[i]);
                 if (   RT_SUCCESS(vrc)
                     && RTDirExists(aDir))
                 {
