@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2010-2012 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,7 +21,11 @@
 #define ____H_PCIDEVICEATTACHMENTIMPL
 
 #include "PCIDeviceAttachmentWrap.h"
-#include <VBox/settings.h>
+
+namespace settings
+{
+    struct HostPCIDeviceAttachment;
+}
 
 class ATL_NO_VTABLE PCIDeviceAttachment :
     public PCIDeviceAttachmentWrap
@@ -32,7 +36,7 @@ public:
 
     // public initializer/uninitializer for internal purposes only
     HRESULT init(IMachine *    aParent,
-                 const Bstr    &aName,
+                 const Utf8Str &aName,
                  LONG          aHostAddess,
                  LONG          aGuestAddress,
                  BOOL          fPhysical);
