@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2010-2014 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,13 +29,14 @@
 class ATL_NO_VTABLE VirtualBoxClient :
     public VirtualBoxClientWrap
 #ifdef RT_OS_WINDOWS
-    , public CComCoClass<VirtualBoxClient, &CLSID_VirtualBoxClient>
+    , public ATL::CComCoClass<VirtualBoxClient, &CLSID_VirtualBoxClient>
 #endif
 {
 public:
-    DECLARE_CLASSFACTORY()
+    DECLARE_CLASSFACTORY_SINGLETON(VirtualBoxClient)
 
-    DECLARE_REGISTRY_RESOURCEID(IDR_VIRTUALBOX)
+    // Do not use any ATL registry support.
+    //DECLARE_REGISTRY_RESOURCEID(IDR_VIRTUALBOX)
 
     DECLARE_NOT_AGGREGATABLE(VirtualBoxClient)
 
@@ -46,6 +47,11 @@ public:
     HRESULT init();
     void uninit();
 
+#ifdef RT_OS_WINDOWS
+    /* HACK ALERT! Implemented in dllmain.cpp. */
+    ULONG InternalRelease();
+#endif
+
 private:
     // wrapped IVirtualBoxClient properties
     virtual HRESULT getVirtualBox(ComPtr<IVirtualBox> &aVirtualBox);
@@ -64,9 +70,19 @@ private:
 
     struct Data
     {
-        Data()
+        Data() : m_ThreadWatcher(NIL_RTTHREAD), m_SemEvWatcher(NIL_RTSEMEVENT)
         {}
 
+        ~Data()
+        {
+            /* HACK ALERT! This is for DllCanUnloadNow(). */
+            if (m_pEventSource.isNotNull())
+            {
+                s_cUnnecessaryAtlModuleLocks--;
+                AssertMsg(s_cUnnecessaryAtlModuleLocks == 0, ("%d\n", s_cUnnecessaryAtlModuleLocks));
+            }
+        }
+
         ComPtr<IVirtualBox> m_pVirtualBox;
         const ComObjPtr<EventSource> m_pEventSource;
 
@@ -75,7 +91,13 @@ private:
     };
 
     Data mData;
+
+public:
+    /** Hack for discounting the AtlModule lock held by Data::m_pEventSource during
+     * DllCanUnloadNow().  This is incremented to 1 when init() initialized
+     * m_pEventSource and is decremented by the Data destructor (above). */
+    static LONG s_cUnnecessaryAtlModuleLocks;
 };
 
-#endif // ____H_VIRTUALBOXCLIENTIMPL
+#endif
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
