@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2014-2015 Oracle Corporation
+ * Copyright (C) 2014-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -42,7 +42,7 @@
 #define CA_STATUS_INIT      UINT32_C(2) /* The device is initialized */
 #define CA_STATUS_IN_UNINIT UINT32_C(3) /* The device is currently uninitializing */
 
-//@todo move t_sample as a PDM interface
+/// @todo move t_sample as a PDM interface
 //typedef struct { int mute;  uint32_t r; uint32_t l; } volume_t;
 
 #define INT_MAX         0x7fffffff
@@ -129,7 +129,7 @@ static DECLCALLBACK(int) drvAudioVideoRecInit(PPDMIHOSTAUDIO pInterface)
 }
 
 /** @todo Replace this with drvAudioHlpPcmPropsFromCfg(). */
-static int drvAudioVideoRecPcmInitInfo(PDMPCMPROPS * pProps, PPDMAUDIOSTREAMCFG as)
+static int drvAudioVideoRecPcmInitInfo(PDMAUDIOPCMPROPS * pProps, PPDMAUDIOSTREAMCFG as)
 {
     int rc = VINF_SUCCESS;
 
@@ -138,21 +138,21 @@ static int drvAudioVideoRecPcmInitInfo(PDMPCMPROPS * pProps, PPDMAUDIOSTREAMCFG
 
     switch (as->enmFormat)
     {
-        case AUD_FMT_S8:
+        case PDMAUDIOFMT_S8:
             fSigned = 1;
-        case AUD_FMT_U8:
+        case PDMAUDIOFMT_U8:
             break;
 
-        case AUD_FMT_S16:
+        case PDMAUDIOFMT_S16:
             fSigned = 1;
-        case AUD_FMT_U16:
+        case PDMAUDIOFMT_U16:
             cBits = 16;
             cShift = 1;
             break;
 
-        case AUD_FMT_S32:
+        case PDMAUDIOFMT_S32:
             fSigned = 1;
-        case AUD_FMT_U32:
+        case PDMAUDIOFMT_U32:
             cBits = 32;
             cShift = 2;
             break;
@@ -168,7 +168,7 @@ static int drvAudioVideoRecPcmInitInfo(PDMPCMPROPS * pProps, PPDMAUDIOSTREAMCFG
     pProps->cChannels = as->cChannels;
     pProps->cShift = (as->cChannels == 2) + cShift;
     pProps->uAlign = (1 << pProps->cShift) - 1;
-    pProps->cbPerSec = pProps->uHz << pProps->cShift;
+    pProps->cbBitrate = (pProps->cBits * pProps->uHz * pProps->cChannels) / 8;
     pProps->fSwapEndian = (as->enmEndianness != PDMAUDIOHOSTENDIANESS);
 
     return rc;
@@ -211,9 +211,9 @@ static int audio_pcm_hw_get_live_out2 (PPDMAUDIOHSTSTRMOUT hw, int *nb_live)
     {
         int live = smin;
 
-        if (live < 0 || live > hw->cSamples)
+        if (live < 0 || live > hw->cSampleBufferSize)
         {
-            LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSamples));
+            LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSampleBufferSize));
             return 0;
         }
         return live;
@@ -227,9 +227,9 @@ static int audio_pcm_hw_get_live_out (PPDMAUDIOHSTSTRMOUT hw)
     int live;
 
     live = audio_pcm_hw_get_live_out2 (hw, &nb_live);
-    if (live < 0 || live > hw->cSamples)
+    if (live < 0 || live > hw->cSampleBufferSize)
     {
-        LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSamples));
+        LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSampleBufferSize));
         return 0;
     }
     return live;
@@ -256,9 +256,9 @@ static int audio_pcm_hw_find_min_in (PPDMAUDIOHSTSTRMIN hw)
 int audio_pcm_hw_get_live_in (PPDMAUDIOHSTSTRMIN hw)
 {
     int live = hw->cTotalSamplesCaptured - audio_pcm_hw_find_min_in (hw);
-    if (live < 0 || live > hw->cSamples)
+    if (live < 0 || live > hw->cSampleBufferSize)
     {
-        LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSamples));
+        LogFlowFunc(("Error: live=%d hw->samples=%d\n", live, hw->cSampleBufferSize));
         return 0;
     }
     return live;
@@ -382,7 +382,7 @@ static DECLCALLBACK(int) drvAudioVideoRecInitOut(PPDMIHOSTAUDIO pInterface, PPDM
     PDRVAUDIOVIDEOREC pDrv = RT_FROM_MEMBER(pInterface, DRVAUDIOVIDEOREC, IHostAudio);
 
     PVIDEORECAUDIOOUT pVRDEVoiceOut = (PVIDEORECAUDIOOUT)pHostVoiceOut;
-    pHostVoiceOut->cSamples = _4K; /* 4096 samples * 4 = 16K bytes total. */
+    pHostVoiceOut->cSampleBufferSize = _4K; /* 4096 samples * 4 = 16K bytes total. */
 
     return drvAudioVideoRecPcmInitInfo(&pVRDEVoiceOut->pHostVoiceOut.Props, pCfg);
 }
@@ -393,7 +393,7 @@ static DECLCALLBACK(int) drvAudioVideoRecInitIn(PPDMIHOSTAUDIO pInterface, PPDMA
     PDRVAUDIOVIDEOREC pDrv = RT_FROM_MEMBER(pInterface, DRVAUDIOVIDEOREC, IHostAudio);
 
     PVIDEORECAUDIOIN pVRDEVoice = (PVIDEORECAUDIOIN)pHostVoiceIn;
-    pHostVoiceIn->cSamples = _4K; /* 4096 samples * 4 = 16K bytes total. */
+    pHostVoiceIn->cSampleBufferSize = _4K; /* 4096 samples * 4 = 16K bytes total. */
 
     return drvAudioVideoRecPcmInitInfo(&pVRDEVoice->pHostVoiceIn.Props, pCfg);
 }
@@ -417,7 +417,7 @@ static DECLCALLBACK(int) drvAudioVideoRecCaptureIn(PPDMIHOSTAUDIO pInterface, PP
     size_t cSamplesRingBuffer = RTCircBufUsed(pVRDEVoice->pRecordedVoiceBuf) / sizeof(PDMAUDIOSAMPLE);
 
     /* How much space is available in the mix buffer. Use the smaller size of the too. */
-    cSamplesRingBuffer = RT_MIN(cSamplesRingBuffer, (uint32_t)(pVRDEVoice->pHostVoiceIn.cSamples -
+    cSamplesRingBuffer = RT_MIN(cSamplesRingBuffer, (uint32_t)(pVRDEVoice->pHostVoiceIn.cSampleBufferSize -
                                 audio_pcm_hw_get_live_in (&pVRDEVoice->pHostVoiceIn)));
 
     LogFlowFunc(("Start reading buffer with %d samples (%d bytes)\n", cSamplesRingBuffer,
@@ -429,14 +429,14 @@ static DECLCALLBACK(int) drvAudioVideoRecCaptureIn(PPDMIHOSTAUDIO pInterface, PP
     {
         /* How much is left? Split request at the end of our samples buffer. */
         size_t cSamplesToRead = RT_MIN(cSamplesRingBuffer - cSamplesRead,
-                                       (uint32_t)(pVRDEVoice->pHostVoiceIn.cSamples - pVRDEVoice->pHostVoiceIn.offSamplesWritten));
+                                       (uint32_t)(pVRDEVoice->pHostVoiceIn.cSampleBufferSize - pVRDEVoice->pHostVoiceIn.offSamplesWritten));
         size_t cbToRead = cSamplesToRead * sizeof(PDMAUDIOSAMPLE);
         LogFlowFunc(("Try reading %zu samples (%zu bytes)\n", cSamplesToRead, cbToRead));
 
         /* Try to acquire the necessary block from the ring buffer. Remeber in fltRecrodCallback we
          * we are filling this buffer with the audio data available from VRDP. Here we are reading it
          */
-        /*todo do I need to introduce a thread to fill the buffer in fltRecordcallback. So that
+        /** @todo do I need to introduce a thread to fill the buffer in fltRecordcallback. So that
          * filling is in separate thread and the reading of that buffer is in separate thread
          */
         void *pvSrc;
@@ -461,7 +461,7 @@ static DECLCALLBACK(int) drvAudioVideoRecCaptureIn(PPDMIHOSTAUDIO pInterface, PP
             break;
 
         pVRDEVoice->pHostVoiceIn.offSamplesWritten = (pVRDEVoice->pHostVoiceIn.offSamplesWritten + cSamplesToRead)
-                                              % pVRDEVoice->pHostVoiceIn.cSamples;
+                                              % pVRDEVoice->pHostVoiceIn.cSampleBufferSize;
 
         /* How much have we reads so far. */
         cSamplesRead += cSamplesToRead;
@@ -501,22 +501,22 @@ static DECLCALLBACK(int) drvAudioVideoRecPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
     LogFlowFunc(("freq=%d, chan=%d, cBits = %d, fsigned = %d, cSamples=%d format=%d\n",
                  pHostVoiceOut->Props.uHz, pHostVoiceOut->Props.cChannels,
                  pHostVoiceOut->Props.cBits, pHostVoiceOut->Props.fSigned,
-                 pHostVoiceOut->cSamples, format));
+                 pHostVoiceOut->cSampleBufferSize, format));
 
     pVRDEVoiceOut->old_ticks = now;
     int cSamplesToSend = RT_MIN(live, cSamplesPlayed);
 
-    if (pHostVoiceOut->cOffSamplesRead + cSamplesToSend > pHostVoiceOut->cSamples)
+    if (pHostVoiceOut->cOffSamplesRead + cSamplesToSend > pHostVoiceOut->cSampleBufferSize)
     {
         /* send the samples till the end of pHostStereoSampleBuf */
         pDrv->pConsoleVRDPServer->SendAudioSamples(&pHostVoiceOut->paSamples[pHostVoiceOut->cOffSamplesRead],
-                                                   (pHostVoiceOut->cSamples - pHostVoiceOut->cOffSamplesRead), format);
+                                                   (pHostVoiceOut->cSampleBufferSize - pHostVoiceOut->cOffSamplesRead), format);
         /*pHostStereoSampleBuff already has the samples which exceeded its space. They have overwriten the old
          * played sampled starting from offset 0. So based on the number of samples that we had to play,
          * read the number of samples from offset 0 .
          */
         pDrv->pConsoleVRDPServer->SendAudioSamples(&pHostVoiceOut->paSamples[0],
-                                                   (cSamplesToSend - (pHostVoiceOut->cSamples -
+                                                   (cSamplesToSend - (pHostVoiceOut->cSampleBufferSize -
                                                                       pHostVoiceOut->cOffSamplesRead)),
                                                    format);
     }
@@ -526,7 +526,7 @@ static DECLCALLBACK(int) drvAudioVideoRecPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
                                                    cSamplesToSend, format);
     }
 
-    pHostVoiceOut->cOffSamplesRead = (pHostVoiceOut->cOffSamplesRead + cSamplesToSend) % pHostVoiceOut->cSamples;
+    pHostVoiceOut->cOffSamplesRead = (pHostVoiceOut->cOffSamplesRead + cSamplesToSend) % pHostVoiceOut->cSampleBufferSize;
 
     *pcSamplesPlayed = cSamplesToSend;
     return VINF_SUCCESS;
@@ -572,7 +572,7 @@ static DECLCALLBACK(int) drvAudioVideoRecControlIn(PPDMIHOSTAUDIO pInterface, PP
     /* initialize only if not already done */
     if (enmStreamCmd == PDMAUDIOSTREAMCMD_ENABLE)
     {
-        //@todo if (!pVRDEVoice->fIsInit)
+        /// @todo if (!pVRDEVoice->fIsInit)
         //    RTCircBufReset(pVRDEVoice->pRecordedVoiceBuf);
         pVRDEVoice->fIsInit = 1;
         pVRDEVoice->pHostVoiceIn = *pHostVoiceIn;
@@ -583,14 +583,14 @@ static DECLCALLBACK(int) drvAudioVideoRecControlIn(PPDMIHOSTAUDIO pInterface, PP
         pVRDEVoice->pvRateBuffer = NULL;
         pVRDEVoice->cbRateBufferAllocated = 0;
 
-        pVRDEVoice->pHostVoiceIn.cSamples = 2048;
+        pVRDEVoice->pHostVoiceIn.cSampleBufferSize = 2048;
         /* Initialize the hardware info section with the audio settings */
 
         ASMAtomicWriteU32(&pVRDEVoice->status, CA_STATUS_IN_INIT);
 
         /* Create the internal ring buffer. */
         RTCircBufCreate(&pVRDEVoice->pRecordedVoiceBuf,
-                        pVRDEVoice->pHostVoiceIn.cSamples * sizeof(PDMAUDIOSAMPLE));
+                        pVRDEVoice->pHostVoiceIn.cSampleBufferSize * sizeof(PDMAUDIOSAMPLE));
 
         if (!RT_VALID_PTR(pVRDEVoice->pRecordedVoiceBuf))
         {
@@ -599,7 +599,7 @@ static DECLCALLBACK(int) drvAudioVideoRecControlIn(PPDMIHOSTAUDIO pInterface, PP
         }
 
         ASMAtomicWriteU32(&pVRDEVoice->status, CA_STATUS_INIT);
-        return pDrv->pConsoleVRDPServer->SendAudioInputBegin(NULL, pVRDEVoice, pHostVoiceIn->cSamples,
+        return pDrv->pConsoleVRDPServer->SendAudioInputBegin(NULL, pVRDEVoice, pHostVoiceIn->cSampleBufferSize,
                                                              pHostVoiceIn->Props.uHz,
                                                              pHostVoiceIn->Props.cChannels, pHostVoiceIn->Props.cBits);
     }
@@ -616,14 +616,17 @@ static DECLCALLBACK(int) drvAudioVideoRecControlIn(PPDMIHOSTAUDIO pInterface, PP
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) drvAudioVideoRecGetConf(PPDMIHOSTAUDIO pInterface, PPDMAUDIOBACKENDCFG pAudioConf)
+static DECLCALLBACK(int) drvAudioVideoRecGetConf(PPDMIHOSTAUDIO pInterface, PPDMAUDIOBACKENDCFG pCfg)
 {
-    LogFlowFunc(("pAudioConf=%p\n", pAudioConf));
+    NOREF(pInterface);
+    AssertPtrReturn(pCfg, VERR_INVALID_POINTER);
 
-    pAudioConf->cbStreamOut = sizeof(VIDEORECAUDIOOUT);
-    pAudioConf->cbStreamIn = sizeof(VIDEORECAUDIOIN);
-    pAudioConf->cMaxHstStrmsOut = 1;
-    pAudioConf->cMaxHstStrmsIn = 1;
+    pCfg->cbStreamIn     = sizeof(VIDEORECAUDIOIN);
+    pCfg->cbStreamOut    = sizeof(VIDEORECAUDIOOUT);
+    pCfg->cMaxStreamsIn  = UINT32_MAx;
+    pCfg->cMaxStreamsOut = UINT32_MAX;
+    pCfg->cSources       = 1;
+    pCfg->cSinks         = 1;
 
     return VINF_SUCCESS;
 }
@@ -700,7 +703,7 @@ int AudioVideoRec::handleVideoRecSvrCmdAudioInputEventBegin(void *pvContext, int
     LogFlowFunc(("Required freq as requested by VRDP Server = %d\n", iSampleHz));
     //if (iSampleHz && iSampleHz != pVRDEVoice->pHostVoiceIn.Props.uFrequency)
     {
-        /* @todo if the above condition is false then pVRDEVoice->uFrequency will remain 0 */
+        /** @todo if the above condition is false then pVRDEVoice->uFrequency will remain 0 */
         /*mpDrv->pUpPort->pfnPrepareAudioConversion(mpDrv->pUpPort, iSampleHz,
                                                   pVRDEVoice->pHostVoiceIn.Props.uFrequency,
                                                   &pVRDEVoice->rate);*/
@@ -733,7 +736,7 @@ int AudioVideoRec::handleVideoRecSvrCmdAudioInputEventData(void *pvContext, cons
     pVRDEVoice->convAudioDevFmtToStSampl(pHostStereoSampleBuf, pvData, cSamples, &videorec_nominal_volume);
 
     /* count of rate adjusted samples */
-    pVRDEVoice->uFrequency = 22100; /* @todo handle this. How pVRDEVoice will get proper value */
+    pVRDEVoice->uFrequency = 22100; /** @todo handle this. How pVRDEVoice will get proper value */
     cConvertedSamples = (cSamples * pVRDEVoice->pHostVoiceIn.Props.uHz) / pVRDEVoice->uFrequency;
     vrdeReallocRateAdjSampleBuf(pVRDEVoice, cConvertedSamples);
 
