@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2014 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -28,41 +28,9 @@
 #include <VBox/settings.h>
 #include <VBox/com/array.h>
 
-struct NATEngineData
-{
-    NATEngineData() : mMtu(0),
-             mSockRcv(0),
-             mSockSnd(0),
-             mTcpRcv(0),
-             mTcpSnd(0),
-             mDNSPassDomain(TRUE),
-             mDNSProxy(FALSE),
-             mDNSUseHostResolver(FALSE),
-             mAliasMode(0)
-    {}
-
-    com::Utf8Str mNetwork;
-    com::Utf8Str mBindIP;
-    uint32_t mMtu;
-    uint32_t mSockRcv;
-    uint32_t mSockSnd;
-    uint32_t mTcpRcv;
-    uint32_t mTcpSnd;
-    /* TFTP service */
-    Utf8Str mTFTPPrefix;
-    Utf8Str mTFTPBootFile;
-    Utf8Str mTFTPNextServer;
-    /* DNS service */
-    BOOL mDNSPassDomain;
-    BOOL mDNSProxy;
-    BOOL mDNSUseHostResolver;
-    /* Alias service */
-    ULONG mAliasMode;
-};
-
 struct NATEngine::Data
 {
-    Backupable<NATEngineData> m;
+    Backupable<settings::NAT> m;
 };
 
 
@@ -89,11 +57,10 @@ HRESULT NATEngine::init(Machine *aParent, INetworkAdapter *aAdapter)
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
     autoInitSpan.setSucceeded();
-    m_fModified = false;
     mData = new Data();
     mData->m.allocate();
-    mData->m->mNetwork.setNull();
-    mData->m->mBindIP.setNull();
+    mData->m->strNetwork.setNull();
+    mData->m->strBindIP.setNull();
     unconst(mParent) = aParent;
     unconst(mAdapter) = aAdapter;
     return S_OK;
@@ -112,12 +79,6 @@ HRESULT NATEngine::init(Machine *aParent, INetworkAdapter *aAdapter, NATEngine *
 
     mData = new Data();
     mData->m.share(aThat->mData->m);
-    NATRuleMap::iterator it;
-    mNATRules.clear();
-    for (it = aThat->mNATRules.begin(); it != aThat->mNATRules.end(); ++it)
-    {
-        mNATRules.insert(std::make_pair(it->first, it->second));
-    }
     unconst(mParent) = aParent;
     unconst(mAdapter) = aAdapter;
     unconst(mPeer) = aThat;
@@ -139,16 +100,11 @@ HRESULT NATEngine::initCopy(Machine *aParent, INetworkAdapter *aAdapter, NATEngi
 
     mData = new Data();
     mData->m.attachCopy(aThat->mData->m);
-    NATRuleMap::iterator it;
-    mNATRules.clear();
-    for (it = aThat->mNATRules.begin(); it != aThat->mNATRules.end(); ++it)
-    {
-        mNATRules.insert(std::make_pair(it->first, it->second));
-    }
     unconst(mAdapter) = aAdapter;
     unconst(mParent) = aParent;
     autoInitSpan.setSucceeded();
-    return BaseFinalConstruct();
+
+    return S_OK;
 }
 
 
@@ -158,7 +114,6 @@ void NATEngine::uninit()
     if (autoUninitSpan.uninitDone())
         return;
 
-    mNATRules.clear();
     mData->m.free();
     delete mData;
     mData = NULL;
@@ -168,27 +123,19 @@ void NATEngine::uninit()
 
 bool NATEngine::i_isModified()
 {
-    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    bool fModified = m_fModified;
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+    bool fModified = mData->m.isBackedUp();
     return fModified;
 }
 
-bool NATEngine::i_rollback()
+void NATEngine::i_rollback()
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturn(autoCaller.rc(), false);
+    AssertComRCReturnVoid(autoCaller.rc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    bool fChanged = m_fModified;
 
-    if (m_fModified)
-    {
-        /* we need to check all data to see whether anything will be changed
-         * after rollback */
-        mData->m.rollback();
-    }
-    m_fModified = false;
-    return fChanged;
+    mData->m.rollback();
 }
 
 void NATEngine::i_commit()
@@ -203,36 +150,48 @@ void NATEngine::i_commit()
     /* lock both for writing since we modify both (mPeer is "master" so locked
      * first) */
     AutoMultiWriteLock2 alock(mPeer, this COMMA_LOCKVAL_SRC_POS);
-    if (m_fModified)
+    if (mData->m.isBackedUp())
     {
         mData->m.commit();
         if (mPeer)
-        {
             mPeer->mData->m.attach(mData->m);
-            mPeer->mNATRules.clear();
-            NATRuleMap::iterator it;
-            for (it = mNATRules.begin(); it != mNATRules.end(); ++it)
-            {
-                mPeer->mNATRules.insert(std::make_pair(it->first, it->second));
-            }
-        }
     }
-    m_fModified = false;
+}
+
+void NATEngine::i_copyFrom(NATEngine *aThat)
+{
+    AssertReturnVoid(aThat != NULL);
+
+    /* sanity */
+    AutoCaller autoCaller(this);
+    AssertComRCReturnVoid(autoCaller.rc());
+
+    /* sanity too */
+    AutoCaller thatCaller(aThat);
+    AssertComRCReturnVoid(thatCaller.rc());
+
+    /* peer is not modified, lock it for reading (aThat is "master" so locked
+     * first) */
+    AutoReadLock rl(aThat COMMA_LOCKVAL_SRC_POS);
+    AutoWriteLock wl(this COMMA_LOCKVAL_SRC_POS);
+
+    /* this will back up current data */
+    mData->m.assignCopy(aThat->mData->m);
 }
 
 HRESULT NATEngine::getNetworkSettings(ULONG *aMtu, ULONG *aSockSnd, ULONG *aSockRcv, ULONG *aTcpWndSnd, ULONG *aTcpWndRcv)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
     if (aMtu)
-        *aMtu = mData->m->mMtu;
+        *aMtu = mData->m->u32Mtu;
     if (aSockSnd)
-        *aSockSnd = mData->m->mSockSnd;
+        *aSockSnd = mData->m->u32SockSnd;
     if (aSockRcv)
-        *aSockRcv = mData->m->mSockRcv;
+        *aSockRcv = mData->m->u32SockRcv;
     if (aTcpWndSnd)
-        *aTcpWndSnd = mData->m->mTcpSnd;
+        *aTcpWndSnd = mData->m->u32TcpSnd;
     if (aTcpWndRcv)
-        *aTcpWndRcv = mData->m->mTcpRcv;
+        *aTcpWndRcv = mData->m->u32TcpRcv;
 
     return S_OK;
 }
@@ -244,21 +203,19 @@ HRESULT NATEngine::setNetworkSettings(ULONG aMtu, ULONG aSockSnd, ULONG aSockRcv
         || aTcpWndSnd || aTcpWndRcv)
     {
         mData->m.backup();
-        m_fModified = true;
+        mParent->i_setModified(Machine::IsModified_NetworkAdapters);
     }
     if (aMtu)
-        mData->m->mMtu = aMtu;
+        mData->m->u32Mtu = aMtu;
     if (aSockSnd)
-        mData->m->mSockSnd = aSockSnd;
+        mData->m->u32SockSnd = aSockSnd;
     if (aSockRcv)
-        mData->m->mSockRcv = aSockSnd;
+        mData->m->u32SockRcv = aSockSnd;
     if (aTcpWndSnd)
-        mData->m->mTcpSnd = aTcpWndSnd;
+        mData->m->u32TcpSnd = aTcpWndSnd;
     if (aTcpWndRcv)
-        mData->m->mTcpRcv = aTcpWndRcv;
+        mData->m->u32TcpRcv = aTcpWndRcv;
 
-    if (m_fModified)
-        mParent->i_setModified(Machine::IsModified_NetworkAdapters);
     return S_OK;
 }
 
@@ -267,10 +224,10 @@ HRESULT NATEngine::getRedirects(std::vector<com::Utf8Str> &aRedirects)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    aRedirects.resize(mNATRules.size());
+    aRedirects.resize(mData->m->mapRules.size());
     size_t i = 0;
-    NATRuleMap::const_iterator it;
-    for (it = mNATRules.begin(); it != mNATRules.end(); ++it, ++i)
+    settings::NATRulesMap::const_iterator it;
+    for (it = mData->m->mapRules.begin(); it != mData->m->mapRules.end(); ++it, ++i)
     {
         settings::NATRule r = it->second;
         aRedirects[i] = Utf8StrFmt("%s,%d,%s,%d,%s,%d",
@@ -305,8 +262,8 @@ HRESULT NATEngine::addRedirect(const com::Utf8Str &aName, NATProtocol_T aProto,
     if (name.isEmpty())
         name = Utf8StrFmt("%s_%d_%d", proto, aHostPort, aGuestPort);
 
-    NATRuleMap::iterator it;
-    for (it = mNATRules.begin(); it != mNATRules.end(); ++it)
+    settings::NATRulesMap::iterator it;
+    for (it = mData->m->mapRules.begin(); it != mData->m->mapRules.end(); ++it)
     {
         r = it->second;
         if (it->first == name)
@@ -319,15 +276,15 @@ HRESULT NATEngine::addRedirect(const com::Utf8Str &aName, NATProtocol_T aProto,
                             tr("A NAT rule for this host port and this host IP already exists"));
     }
 
+    mData->m.backup();
     r.strName = name.c_str();
     r.proto = aProto;
     r.strHostIP = aHostIP;
     r.u16HostPort = aHostPort;
     r.strGuestIP = aGuestIP;
     r.u16GuestPort = aGuestPort;
-    mNATRules.insert(std::make_pair(name, r));
+    mData->m->mapRules.insert(std::make_pair(name, r));
     mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-    m_fModified = true;
 
     ULONG ulSlot;
     mAdapter->COMGETTER(Slot)(&ulSlot);
@@ -341,26 +298,24 @@ HRESULT NATEngine::addRedirect(const com::Utf8Str &aName, NATProtocol_T aProto,
 HRESULT NATEngine::removeRedirect(const com::Utf8Str &aName)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    NATRuleMap::iterator it = mNATRules.find(aName);
-    if (it == mNATRules.end())
+    settings::NATRulesMap::iterator it = mData->m->mapRules.find(aName);
+    if (it == mData->m->mapRules.end())
         return E_INVALIDARG;
     mData->m.backup();
+    /*
+     * NB: "it" may now point to the backup!  In that case it's ok to
+     * get data from the backup copy of s.mapRules via it, but we can't
+     * erase(it) from potentially new s.mapRules.
+     */
     settings::NATRule r = it->second;
-    Utf8Str strHostIP = r.strHostIP;
-    Utf8Str strGuestIP = r.strGuestIP;
-    NATProtocol_T proto = r.proto;
-    uint16_t u16HostPort = r.u16HostPort;
-    uint16_t u16GuestPort = r.u16GuestPort;
     ULONG ulSlot;
     mAdapter->COMGETTER(Slot)(&ulSlot);
 
-    mNATRules.erase(it);
+    mData->m->mapRules.erase(aName); /* NB: erase by key, "it" may not be valid */
     mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-    m_fModified = true;
-    mData->m.commit();
     alock.release();
-    mParent->i_onNATRedirectRuleChange(ulSlot, TRUE, Bstr(aName).raw(), proto, Bstr(strHostIP).raw(),
-                                       u16HostPort, Bstr(strGuestIP).raw(), u16GuestPort);
+    mParent->i_onNATRedirectRuleChange(ulSlot, TRUE, Bstr(aName).raw(), r.proto, Bstr(r.strHostIP).raw(),
+                                       r.u16HostPort, Bstr(r.strGuestIP).raw(), r.u16GuestPort);
     return S_OK;
 }
 
@@ -370,34 +325,8 @@ HRESULT NATEngine::i_loadSettings(const settings::NAT &data)
     AssertComRCReturnRC(autoCaller.rc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    HRESULT rc = S_OK;
-    mData->m->mNetwork = data.strNetwork;
-    mData->m->mBindIP = data.strBindIP;
-    mData->m->mMtu = data.u32Mtu;
-    mData->m->mSockSnd = data.u32SockSnd;
-    mData->m->mTcpRcv = data.u32TcpRcv;
-    mData->m->mTcpSnd = data.u32TcpSnd;
-    /* TFTP */
-    mData->m->mTFTPPrefix = data.strTFTPPrefix;
-    mData->m->mTFTPBootFile = data.strTFTPBootFile;
-    mData->m->mTFTPNextServer = data.strTFTPNextServer;
-    /* DNS */
-    mData->m->mDNSPassDomain = data.fDNSPassDomain;
-    mData->m->mDNSProxy = data.fDNSProxy;
-    mData->m->mDNSUseHostResolver = data.fDNSUseHostResolver;
-    /* Alias */
-    mData->m->mAliasMode  = (data.fAliasUseSamePorts ? NATAliasMode_AliasUseSamePorts : 0);
-    mData->m->mAliasMode |= (data.fAliasLog          ? NATAliasMode_AliasLog          : 0);
-    mData->m->mAliasMode |= (data.fAliasProxyOnly    ? NATAliasMode_AliasProxyOnly    : 0);
-    /* port forwarding */
-    mNATRules.clear();
-    for (settings::NATRuleList::const_iterator it = data.llRules.begin();
-        it != data.llRules.end(); ++it)
-    {
-        mNATRules.insert(std::make_pair(it->strName, *it));
-    }
-    m_fModified = false;
-    return rc;
+    mData->m.assignCopy(&data);
+    return S_OK;
 }
 
 
@@ -408,42 +337,18 @@ HRESULT NATEngine::i_saveSettings(settings::NAT &data)
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
     HRESULT rc = S_OK;
-    data.strNetwork = mData->m->mNetwork;
-    data.strBindIP = mData->m->mBindIP;
-    data.u32Mtu = mData->m->mMtu;
-    data.u32SockRcv = mData->m->mSockRcv;
-    data.u32SockSnd = mData->m->mSockSnd;
-    data.u32TcpRcv = mData->m->mTcpRcv;
-    data.u32TcpSnd = mData->m->mTcpSnd;
-    /* TFTP */
-    data.strTFTPPrefix = mData->m->mTFTPPrefix;
-    data.strTFTPBootFile = mData->m->mTFTPBootFile;
-    data.strTFTPNextServer = mData->m->mTFTPNextServer;
-    /* DNS */
-    data.fDNSPassDomain = !!mData->m->mDNSPassDomain;
-    data.fDNSProxy = !!mData->m->mDNSProxy;
-    data.fDNSUseHostResolver = !!mData->m->mDNSUseHostResolver;
-    /* Alias */
-    data.fAliasLog = !!(mData->m->mAliasMode & NATAliasMode_AliasLog);
-    data.fAliasProxyOnly = !!(mData->m->mAliasMode & NATAliasMode_AliasProxyOnly);
-    data.fAliasUseSamePorts = !!(mData->m->mAliasMode & NATAliasMode_AliasUseSamePorts);
-
-    for (NATRuleMap::iterator it = mNATRules.begin();
-        it != mNATRules.end(); ++it)
-        data.llRules.push_back(it->second);
-    m_fModified = false;
+    data = *mData->m.data();
     return rc;
 }
 
 HRESULT NATEngine::setNetwork(const com::Utf8Str &aNetwork)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (Bstr(mData->m->mNetwork) != aNetwork)
+    if (mData->m->strNetwork != aNetwork)
     {
         mData->m.backup();
-        mData->m->mNetwork = aNetwork;
+        mData->m->strNetwork = aNetwork;
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -452,10 +357,10 @@ HRESULT NATEngine::setNetwork(const com::Utf8Str &aNetwork)
 HRESULT NATEngine::getNetwork(com::Utf8Str &aNetwork)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (!mData->m->mNetwork.isEmpty())
+    if (!mData->m->strNetwork.isEmpty())
     {
-        aNetwork = mData->m->mNetwork;
-        Log(("Getter (this:%p) Network: %s\n", this, mData->m->mNetwork.c_str()));
+        aNetwork = mData->m->strNetwork;
+        Log(("Getter (this:%p) Network: %s\n", this, mData->m->strNetwork.c_str()));
     }
     return S_OK;
 }
@@ -463,12 +368,11 @@ HRESULT NATEngine::getNetwork(com::Utf8Str &aNetwork)
 HRESULT NATEngine::setHostIP(const com::Utf8Str &aHostIP)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (Bstr(mData->m->mBindIP) != aHostIP)
+    if (mData->m->strBindIP != aHostIP)
     {
         mData->m.backup();
-        mData->m->mBindIP = aHostIP;
+        mData->m->strBindIP = aHostIP;
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -477,20 +381,19 @@ HRESULT NATEngine::getHostIP(com::Utf8Str &aBindIP)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (!mData->m->mBindIP.isEmpty())
-        aBindIP = mData->m->mBindIP;
+    if (!mData->m->strBindIP.isEmpty())
+        aBindIP = mData->m->strBindIP;
     return S_OK;
 }
 
 HRESULT NATEngine::setTFTPPrefix(const com::Utf8Str &aTFTPPrefix)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (Bstr(mData->m->mTFTPPrefix) != aTFTPPrefix)
+    if (mData->m->strTFTPPrefix != aTFTPPrefix)
     {
         mData->m.backup();
-        mData->m->mTFTPPrefix = aTFTPPrefix;
+        mData->m->strTFTPPrefix = aTFTPPrefix;
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -500,10 +403,10 @@ HRESULT NATEngine::getTFTPPrefix(com::Utf8Str &aTFTPPrefix)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (!mData->m->mTFTPPrefix.isEmpty())
+    if (!mData->m->strTFTPPrefix.isEmpty())
     {
-        aTFTPPrefix = mData->m->mTFTPPrefix;
-        Log(("Getter (this:%p) TFTPPrefix: %s\n", this, mData->m->mTFTPPrefix.c_str()));
+        aTFTPPrefix = mData->m->strTFTPPrefix;
+        Log(("Getter (this:%p) TFTPPrefix: %s\n", this, mData->m->strTFTPPrefix.c_str()));
     }
     return S_OK;
 }
@@ -511,12 +414,11 @@ HRESULT NATEngine::getTFTPPrefix(com::Utf8Str &aTFTPPrefix)
 HRESULT NATEngine::setTFTPBootFile(const com::Utf8Str &aTFTPBootFile)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (Bstr(mData->m->mTFTPBootFile) != aTFTPBootFile)
+    if (mData->m->strTFTPBootFile != aTFTPBootFile)
     {
         mData->m.backup();
-        mData->m->mTFTPBootFile = aTFTPBootFile;
+        mData->m->strTFTPBootFile = aTFTPBootFile;
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -525,10 +427,10 @@ HRESULT NATEngine::setTFTPBootFile(const com::Utf8Str &aTFTPBootFile)
 HRESULT NATEngine::getTFTPBootFile(com::Utf8Str &aTFTPBootFile)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (!mData->m->mTFTPBootFile.isEmpty())
+    if (!mData->m->strTFTPBootFile.isEmpty())
     {
-        aTFTPBootFile = mData->m->mTFTPBootFile;
-        Log(("Getter (this:%p) BootFile: %s\n", this, mData->m->mTFTPBootFile.c_str()));
+        aTFTPBootFile = mData->m->strTFTPBootFile;
+        Log(("Getter (this:%p) BootFile: %s\n", this, mData->m->strTFTPBootFile.c_str()));
     }
     return S_OK;
 }
@@ -537,12 +439,11 @@ HRESULT NATEngine::getTFTPBootFile(com::Utf8Str &aTFTPBootFile)
 HRESULT NATEngine::setTFTPNextServer(const com::Utf8Str &aTFTPNextServer)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (Bstr(mData->m->mTFTPNextServer) != aTFTPNextServer)
+    if (mData->m->strTFTPNextServer != aTFTPNextServer)
     {
         mData->m.backup();
-        mData->m->mTFTPNextServer = aTFTPNextServer;
+        mData->m->strTFTPNextServer = aTFTPNextServer;
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -550,10 +451,10 @@ HRESULT NATEngine::setTFTPNextServer(const com::Utf8Str &aTFTPNextServer)
 HRESULT NATEngine::getTFTPNextServer(com::Utf8Str &aTFTPNextServer)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-    if (!mData->m->mTFTPNextServer.isEmpty())
+    if (!mData->m->strTFTPNextServer.isEmpty())
     {
-        aTFTPNextServer =  mData->m->mTFTPNextServer;
-        Log(("Getter (this:%p) NextServer: %s\n", this, mData->m->mTFTPNextServer.c_str()));
+        aTFTPNextServer =  mData->m->strTFTPNextServer;
+        Log(("Getter (this:%p) NextServer: %s\n", this, mData->m->strTFTPNextServer.c_str()));
     }
     return S_OK;
 }
@@ -563,12 +464,11 @@ HRESULT NATEngine::setDNSPassDomain(BOOL aDNSPassDomain)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (mData->m->mDNSPassDomain != aDNSPassDomain)
+    if (mData->m->fDNSPassDomain != RT_BOOL(aDNSPassDomain))
     {
         mData->m.backup();
-        mData->m->mDNSPassDomain = aDNSPassDomain;
+        mData->m->fDNSPassDomain = RT_BOOL(aDNSPassDomain);
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -576,7 +476,7 @@ HRESULT NATEngine::setDNSPassDomain(BOOL aDNSPassDomain)
 HRESULT NATEngine::getDNSPassDomain(BOOL *aDNSPassDomain)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-    *aDNSPassDomain = mData->m->mDNSPassDomain;
+    *aDNSPassDomain = mData->m->fDNSPassDomain;
     return S_OK;
 }
 
@@ -585,12 +485,11 @@ HRESULT NATEngine::setDNSProxy(BOOL aDNSProxy)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (mData->m->mDNSProxy != aDNSProxy)
+    if (mData->m->fDNSProxy != RT_BOOL(aDNSProxy))
     {
         mData->m.backup();
-        mData->m->mDNSProxy = aDNSProxy;
+        mData->m->fDNSProxy = RT_BOOL(aDNSProxy);
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -598,7 +497,7 @@ HRESULT NATEngine::setDNSProxy(BOOL aDNSProxy)
 HRESULT NATEngine::getDNSProxy(BOOL *aDNSProxy)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-    *aDNSProxy = mData->m->mDNSProxy;
+    *aDNSProxy = mData->m->fDNSProxy;
     return S_OK;
 }
 
@@ -606,19 +505,18 @@ HRESULT NATEngine::getDNSProxy(BOOL *aDNSProxy)
 HRESULT NATEngine::getDNSUseHostResolver(BOOL *aDNSUseHostResolver)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    *aDNSUseHostResolver = mData->m->mDNSUseHostResolver;
+    *aDNSUseHostResolver = mData->m->fDNSUseHostResolver;
     return S_OK;
 }
 
 
 HRESULT NATEngine::setDNSUseHostResolver(BOOL aDNSUseHostResolver)
 {
-    if (mData->m->mDNSUseHostResolver != aDNSUseHostResolver)
+    if (mData->m->fDNSUseHostResolver != RT_BOOL(aDNSUseHostResolver))
     {
         mData->m.backup();
-        mData->m->mDNSUseHostResolver = aDNSUseHostResolver;
+        mData->m->fDNSUseHostResolver = RT_BOOL(aDNSUseHostResolver);
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -626,13 +524,16 @@ HRESULT NATEngine::setDNSUseHostResolver(BOOL aDNSUseHostResolver)
 HRESULT NATEngine::setAliasMode(ULONG aAliasMode)
 {
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-    if (mData->m->mAliasMode != aAliasMode)
+    ULONG uAliasMode = (mData->m->fAliasUseSamePorts ? NATAliasMode_AliasUseSamePorts : 0);
+    uAliasMode |= (mData->m->fAliasLog ? NATAliasMode_AliasLog : 0);
+    uAliasMode |= (mData->m->fAliasProxyOnly ? NATAliasMode_AliasProxyOnly : 0);
+    if (uAliasMode != aAliasMode)
     {
         mData->m.backup();
-        mData->m->mAliasMode = aAliasMode;
+        mData->m->fAliasUseSamePorts = RT_BOOL(aAliasMode & NATAliasMode_AliasUseSamePorts);
+        mData->m->fAliasLog = RT_BOOL(aAliasMode & NATAliasMode_AliasLog);
+        mData->m->fAliasProxyOnly = RT_BOOL(aAliasMode & NATAliasMode_AliasProxyOnly);
         mParent->i_setModified(Machine::IsModified_NetworkAdapters);
-        m_fModified = true;
     }
     return S_OK;
 }
@@ -640,7 +541,10 @@ HRESULT NATEngine::setAliasMode(ULONG aAliasMode)
 HRESULT NATEngine::getAliasMode(ULONG *aAliasMode)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-    *aAliasMode = mData->m->mAliasMode;
+    ULONG uAliasMode = (mData->m->fAliasUseSamePorts ? NATAliasMode_AliasUseSamePorts : 0);
+    uAliasMode |= (mData->m->fAliasLog ? NATAliasMode_AliasLog : 0);
+    uAliasMode |= (mData->m->fAliasProxyOnly ? NATAliasMode_AliasProxyOnly : 0);
+    *aAliasMode = uAliasMode;
     return S_OK;
 }
 
